#!/usr/bin/perl -w

# $Id: gateway.pl,v 1.1.1.1.2.8 2001/11/28 17:29:08 justin Exp $ #

%config = (database_host => 'localhost',
           database_name => 'ticketsmith',
           database_user => 'ticketsmith',
           database_pass => 'ticketsmith');

# send email report upon receipt (1 = yes, 0 = no)
$send_email_report = 1;

# address to send report to
$report_to_address = "ticketsmith-report\@foo.bar";

# report from address
$report_from_address = "ticketsmith-report\@foo.bar";

# location of sendmail
$mailprog = "/usr/sbin/sendmail";

######################## </CONFIGURATION SECTION> ##############################

# database bindings
use DBI;

# read in message
while (<STDIN>) {
    push @message, $_;
}

# main program
&get_headers();
&check_attachments();
&get_body();
&insert_message();
&mail_report() if ($send_email_report);

exit();

################################################################################

sub get_headers {

    # read in headers
    foreach (@message) {
        push @headers, $_;
        last if (/^\s$/ || /^$/);
        if (/oundary=/) {
            $attachment_info = $_;
            $attachment = 1;
	    }
        else {
            $attachment = 0;
        }
        $_ =~ s/:\s/:/g;
        if (/:/) {
            @vars = split(':', $_, 2);
            if ($vars[1]) {
                chop($header{$vars[0]} = $vars[1]);
            }
        }
    }
    
    # strip out Re:'s in subject
    $header{'Subject'} =~ s/\s*Re:\s*//gi if ($header{'Subject'});
    
    # put a nice Re: back in
    $header{'Subject'} =~ s/(\[\#\d+\])(.*)/$1 Re: $2/ if ($header{'Subject'});

    # initialize Cc: header
    $header{'Cc'} = "" if (!$header{'Cc'});
    
    # fix quoting in email headers
    $header{'From'} =~ s/"/\"/g;
    $header{'Cc'} =~ s/"/\"/g;

    # fix empty subject header
    $header{'Subject'} = "(no subject)" if (!$header{'Subject'});
    
    # determine ticket number
    $parent = $header{'Subject'};
    if ($parent =~ /\[\#(\d+)\]/) {
        $parent =~ s/.*\[\#(\d+)\].*/$1/;
    }
    else {
        $parent = 0;
    }
    
}

################################################################################

sub check_attachments {

    # check for attachment
    return if (!$attachment_info);

    # determine attachment delimiter
    ($i, $boundary) = split("\"", $attachment_info);
    return if (!$boundary);

    # pull out attachments
    for ($i = $#headers + 1; $i <= $#message; $i++) {
        if ($message[$i] =~ /$boundary/) {
            push @boundary_lines, $i;
        }
    }

}

################################################################################

sub get_body {

    # read in message body
    if (!$attachment_info) {
        for ($i = $#headers + 1; $i <= $#message; $i++) {
	    $body .= $message[$i];
        }
    }
    else {
        for ($i = $boundary_lines[0] + 1; $i < $boundary_lines[1]; $i++) {
            if ($past_info) {            
                $body .= $message[$i];
            }
            elsif ($message[$i] =~ /^\s+$/) {
                $past_info = 1;
            }
	}
    }

    $body = "[empty message body]" if (!$body);
    
}

################################################################################

sub insert_message {

    # connect to database
    $dbh = DBI->connect("DBI:mysql:$config{'database_name'}:$config{'database_host'}", $config{'database_user'}, $config{'database_pass'});
    
    # update parent activity
    if ($parent) {
        $activity_query = "UPDATE tickets SET type = 'Open', activity = UNIX_TIMESTAMP() WHERE ticket = '$parent'";
        $sth = $dbh->prepare($activity_query);
        $sth->execute();
        $sth->finish();
        $type = "Client Followup";
        $assignment = "9999";
    }
    else {
        $type = "Open";
        $assignment = "0";
    }

    # quote all fields
    $parent = $dbh->quote($parent);
    $attachment = $dbh->quote($attachment);
    $author = $dbh->quote($header{'From'});
    $subject = $dbh->quote($header{'Subject'});
    $body = $dbh->quote($body);
    $type = $dbh->quote($type);
    $cc = $dbh->quote($header{'Cc'});
    $assignment = $dbh->quote($assignment);

    # do insertion
    $insert_query = "INSERT INTO tickets (parent, attachment, timestamp, author, subject, body, type, cc, assignment) ";
    $insert_query .= "VALUES ($parent, $attachment, UNIX_TIMESTAMP(), $author, $subject, $body, $type, $cc, $assignment)";
    $sth = $dbh->prepare($insert_query);
    $sth->execute();
    $ticket = $sth->{'mysql_insertid'};
    $sth->finish();
    $dbh->disconnect();
		
}

################################################################################

sub mail_report {

    # unquote necessary fields
    $author =~ s/^\'(.*)\'$/$1/;
    $author =~ s/\\\'/'/g;
    $subject =~ s/^\'(.*)\'$/$1/;
    $subject =~ s/\\\'/'/g;
    
    # remove ticket number
    $subject =~ s/\[\#\d+\](.*)/$1/;
    
    # mail the report
    open(MAIL, "|$mailprog -t");
    print MAIL "To: $report_to_address\n";
    print MAIL "From: $report_from_address\n";
    print MAIL "Subject: New support ticket #$ticket\n\n";
    print MAIL "There is a new support ticket:\n\n";
    print MAIL "\tAuthor: $author\n";
    print MAIL "\tSubject: $subject\n";
    close(MAIL);

}
